function [] = Pencoder(cur,arg,h)
%function [] = Pencoder(cur_str,prev_str,framenumber,JP2PQuality,criteria,h)
%Load images for encoding

A=double(imread(strcat('encoder\ppm\',num2str(cur-1),'.pgm')));
B=double(imread(strcat(arg.prefix,num2str(cur),arg.postfix)));
waitbar(0,h,'Calculating vectors...');

%In-loop denoise filter
if (arg.wiener==1)
    A=wiener2(A);
elseif (arg.medfilt==1)
    A=medfilt2(A,[5,5]);
end

%Calculate OF vectors
if (arg.fromOriginal==1)
    Amarked=double(imread(strcat(arg.prefix,num2str(cur-1),arg.postfix)));
    V = OF_vectors(B,Amarked,arg,cur);
else
    V = OF_vectors(B,A,arg,cur);
end

save(strcat('encoder\vector\p',num2str(cur),'.mat'),'V');

%Resize vectorfelt:
if (arg.OFresize~=0)
    V=imresize(V,(arg.OFresize),'bicubic');
    V=round(V);
end
if (arg.MSEadjust ==1)
    %Threshold for when a MSE vector should be calculated
    if (arg.pixelres ~=1)
       error('MSEadjust can only be used with pixelres=1'); 
    end
    VE_threshold=0;
    V=mse_vectors(B,A,arg.search_width,arg.search_height,h,1,V,VE_threshold);
    save(strcat('encoder\vector\p',num2str(cur),'.mat'),'V');
end

V=V./arg.pixelres;


%Calculate estimation error from vectors
waitbar(0,h,'Calculating estimation error');
Estimation=motion_estimation(A,V,h,arg);

%Calculate residual frame
C=double(B)-Estimation;

%Visualizing estimation error

F=abs(C)./60;
imwrite(F,strcat('encoder\ppm\abserror',num2str(cur),'.pgm'));


%Calculating scaling factor
res=arg.res_factor;
if (arg.auto_res==1 && arg.eightbit==1)
    res=abs(127/max(max(C)));
    if (res < abs(127/min(min(C))))
        res=abs(127/min(min(C)));
    end
    res=ceil(res*10)/10;
    if (res>1)
        res=1;
    end
    save(strcat('encoder\argres\res',num2str(cur),'.mat'),'res');
end

%Scaling
C=C.*res;

if (arg.eightbit==1)
    C=uint8(C+127);
else
    C=uint16(C+32768);
end

%Saving estimation error
waitbar(0,h,'Saving estimation error');
if (arg.eightbit==1)
    imwrite(C,strcat('encoder\ppm\p',num2str(cur),'.pgm'));
else
    imwrite(uint8(C-32640),strcat('encoder\ppm\p',num2str(cur),'.pgm'));
end

waitbar(0.25,h);

%DISTORTIONCONTROL

high_q=0.99;
low_q=1.01;
y_min_margin=0.0;
y_max_margin=0.0;


if(arg.eightbit==1)
    y_compressionrato=10;
else
    y_compressionrato=20;
end
orginal=imread(strcat(arg.prefix,num2str(cur),arg.postfix));
imwrite(orginal(:,:,1),strcat('encoder\jp2\tmp\i',num2str(cur),'.jp2'),'CompressionRatio',y_compressionrato);
jp2=imread(strcat('encoder\jp2\tmp\i',num2str(cur),'.jp2'));

jp2PSNR=metrix_mux(orginal,jp2,'psnr');

if(arg.distortioncontrol)
   y_compressionrato=600;
end

fprintf('Distortioncontrol ');
imwrite(C,strcat('encoder\jp2\p_',num2str(cur),'.jp2'),'CompressionRatio',y_compressionrato,'TileSize',arg.tilesize,'ReductionLevels',arg.reductionlevels);
while(arg.distortioncontrol)
    image_tmp=double(imread(strcat('encoder\jp2\p_',num2str(cur),'.jp2')));
    if (arg.eightbit==1)
        image_tmp=image_tmp-127;
    else
        image_tmp=image_tmp-32768;
    end
    image_tmp=round(image_tmp./res);
    image_tmp=uint8(image_tmp+Estimation);
    psnr_y=metrix_mux(orginal,image_tmp,'psnr');
    %y_compressionrato
    if ((psnr_y-jp2PSNR)>y_min_margin)
        break;
    else
        y_compressionrato=y_compressionrato*0.7;
    end
    if(y_compressionrato>600 || y_compressionrato<1)
        break;
    end
    imwrite(C,strcat('encoder\jp2\p_',num2str(cur),'.jp2'),'CompressionRatio',y_compressionrato,'TileSize',arg.tilesize,'ReductionLevels',arg.reductionlevels);
    fprintf('-y');
end

while(arg.distortioncontrol)
    image_tmp=imread(strcat('encoder\jp2\p_',num2str(cur),'.jp2'));
    if (arg.eightbit==1)
        image_tmp=double(image_tmp)-127;
    else
        image_tmp=double(image_tmp)-32768;
    end
    image_tmp=round(image_tmp./res);
    image_tmp=uint8(image_tmp+Estimation);
    psnr_y=metrix_mux(orginal,image_tmp,'psnr');
    %y_compressionrato
    if ((psnr_y-jp2PSNR)<y_max_margin)
        break; 
    elseif ((psnr_y-jp2PSNR)>0)
        y_compressionrato=y_compressionrato*low_q;
    else
        y_compressionrato=y_compressionrato*high_q;
    end
    if(y_compressionrato>600 || y_compressionrato<1)
        break;
    end
    imwrite(C,strcat('encoder\jp2\p_',num2str(cur),'.jp2'),'CompressionRatio',y_compressionrato,'TileSize',arg.tilesize,'ReductionLevels',arg.reductionlevels);
    fprintf('-y');
end
while(arg.distortioncontrol)
    image_tmp=imread(strcat('encoder\jp2\p_',num2str(cur),'.jp2'));
    if (arg.eightbit==1)
        image_tmp=double(image_tmp)-127;
    else
        image_tmp=double(image_tmp)-32768;
    end
    image_tmp=round(image_tmp./res);
    image_tmp=uint8(image_tmp+Estimation);
    psnr_y=metrix_mux(orginal,image_tmp,'psnr');
    %y_compressionrato
    if ((psnr_y-jp2PSNR)>y_min_margin)
        break;
    else
        y_compressionrato=y_compressionrato*high_q;
    end
    if(y_compressionrato>600 || y_compressionrato<1)
        break;
    end
    imwrite(C,strcat('encoder\jp2\p_',num2str(cur),'.jp2'),'CompressionRatio',y_compressionrato,'TileSize',arg.tilesize,'ReductionLevels',arg.reductionlevels);
    fprintf('-y');
end
waitbar(0.5,h);

fprintf('\n');












