%Run this file to encode and decode the two frames from the bunny sequence.
%Decoded sequence is located in encoder/ppm/ with no prefix i.e. 1.pgm and
%2.pgm
%The encoded sequence is located in ecoder/jp2 where reference frames is located
%in encoder/jp2/tmp folder
%Remember to include flow_code and metrix_mux in Matlab path. Refer to
%documentation in their respective folders.



%Perform encoding
encode=true;
%Only perform quality check, PSNR and SSIM
onlyqualitytest=false;
%Number of frames to be encoded
numberofframes=2;
%GOP size
GOP=2;
%Startframe
startframe=1;
%Prefix for pgm images, for example= 'foldername\'
prefix='';

%Postfix - only pgm supported
postfix='.pgm';

%reductionlevels: Number of wavelet reductions, use 6.

%tilesize: Tilesize of 1920,1080, change if material above 1920x1080

%fromOriginal: Calculate OF from uncompressed frame, instead of decoded

%distortioncontrol: 1=on 0=off

%eightbit: 1=8bit residue 0=16 bit residue

%auto_res: Used with 8-bit, scales pixel values in residue automatically to
%[0,255]

%res_factor: 1=Manually change scaling factor

%MSEadjust: 1=Adjust OF vectors with MSE-search

%search_height: Search height/2= Number of pixels searched to each side,
%i.e. 2 will search one above, and one below. Only use even numbers

%search_width: Search width/2= Number of pixels searched to each side,
%i.e. 2 will search one left, and one right. Only use even numbers

%OFresize: Resize of image before OF is calculated. 2= half resolution in
%width, and height.

%pixelres: Pixel accuracy supported: 1=one pixel, 2= half pixel 4= quarter
%pixel

%interpolation: Interpolation method used to calculate sub-pixel accuracy

%wiener: 1=in-loop winer filter is activated 0=off

%medfilt: 1=in-loop median filter is activated 0=off

arg=struct('reductionlevels',6,'tilesize',[1920 1080],'fromOriginal',1, ...
    'distortioncontrol',0,'eightbit',1,'auto_res',1,'res_factor',1,'MSEadjust',0, ...
    'search_height',2,'search_width',2,'OFresize',8, ...
    'pixelres',2,'interpolation','bilinear','wiener',0,'medfilt',0, ...
    'postfix',postfix,'prefix',prefix,'startframe',startframe);


numGOP=ceil(numberofframes/GOP);

%I-frame
h=waitbar(0,'');
h2=waitbar(0,'Total progress');
for c=1:numGOP
    Iframe=startframe+((c-1)*GOP);
    if (c==numGOP)
        if (mod(numberofframes,GOP)==0)
            numPframes=GOP-1;
        else
            numPframes=mod(numberofframes,GOP)-1;
        end
    else
        numPframes=GOP-1;
    end
    
    if not(onlyqualitytest)
           
        waitbar(0,h,'Compressing I-frame');
        if (encode)
            orginal=imread(strcat(prefix,num2str(Iframe),postfix));
            imwrite(orginal,strcat('encoder\jp2\i',num2str(Iframe),'.jp2'),'jp2','CompressionRatio',10);
            imwrite(orginal,strcat('encoder\jp2\tmp\i',num2str(Iframe),'.jp2'),'jp2','CompressionRatio',10);            
        end
        waitbar(0.5,h);
        orginal(:,:)=imread(strcat('encoder\jp2\tmp\i',num2str(Iframe),'.jp2'));
        imwrite(orginal,strcat('encoder\ppm\',num2str(Iframe),'.pgm'));
       
        %P-frames
        waitbar(1,h);
        I=imread(strcat('encoder\ppm\',num2str(Iframe),'.pgm'));
        [height,width,depth]=size(I);
        for cur=Iframe+1:Iframe+numPframes
            tic
            waitbar((cur-startframe)/numberofframes,h2);
            if (encode)
                Pencoder(cur,arg,h)
            end
            Pdecoder(cur,arg,h);
            toc
        end
    end

end


%Calculate SSIM/PSNR

PSNR=zeros(2,numberofframes);
SSIM=zeros(2,numberofframes);
waitbar(0,h,'Calculating PSNR and SSIM');
for x=1:numberofframes
    y=x+startframe-1;
    waitbar(((x-1)*4+1)/((numberofframes-1)*4),h);
    orginal=imread(strcat(arg.prefix,num2str(y),arg.postfix));
    jp2(:,:,1)=imread(strcat('encoder\jp2\tmp\i',num2str(y),'.jp2'));
    jp2diff=imread(strcat('encoder\ppm\',num2str(y),'.pgm'));
    waitbar(((x-1)*4+2)/((numberofframes-1)*4),h);
    PSNR(1,x)=metrix_mux(orginal,jp2diff,'psnr');
    PSNR(2,x)=metrix_mux(orginal,jp2,'psnr');
    waitbar(((x-1)*4+3)/((numberofframes-1)*4),h);
    SSIM(1,x)=metrix_mux(orginal,jp2diff,'ssim');
    SSIM(2,x)=metrix_mux(orginal,jp2,'ssim');
    waitbar(((x-1)*4+4)/((numberofframes-1)*4),h);
end
waitbar(1,h,'Finished!');
PSNR
SSIM
mean(PSNR')
mean(SSIM')
pause(0.1);
save('encoder\workspace.mat');
if (numberofframes==240)
    parse_results(PSNR,SSIM,'encoder\','results.csv','',1,1,1);
end
close(h);
close(h2);