function [] = parse_results(PSNR,SSIM,folder,filename,file_descripion,CSV,SSIMerrorbar,PSNRerrorbar)
if(size(PSNR) ~= [2 ,240])
    warning('PSNR has wrong size, must be [2,240]');
elseif(size(SSIM) ~= [2 ,240])
    warning('SSIM has wrong size, must be [2,240]');
else
    
    %Other values not supported without changing code
    frames_pr_sequence=15;
    number_of_sequences=8;
    
    % 13 degrees of freedom - Students t
    confidence_factor_sequence= ...
        2.145/sqrt(frames_pr_sequence);
    
    % 119 degrees of freedom - Students t
    confidence_factor_total= ...
        1.980/sqrt(frames_pr_sequence*number_of_sequences);
    
    sequence_names=char('Bunny', 'Crowdrun', 'Ducks', ...
        'IntoTree', 'OldTown', 'Parkjoy', 'Vegies', 'Horse');
    
    PSNR_calc=zeros(4,number_of_sequences+1);
    SSIM_calc=zeros(4,number_of_sequences+1);
    
    %PSNR Calculation
    PSNR=10.^(PSNR./10);
    PSNR_tmp=PSNR(1,:)./PSNR(2,:);
    PSNR_tmp=PSNR_tmp(2:2:end);
    
    for x=1:number_of_sequences;
        PSNR_calc(1,x)=mean(PSNR_tmp((x-1)*frames_pr_sequence+1 : ...
            x*frames_pr_sequence));
        
        PSNR_calc(4,x)=std(PSNR_tmp((x-1)*frames_pr_sequence+1 : ...
            x*frames_pr_sequence));
        
        PSNR_calc(2,x)=PSNR_calc(1,x)-confidence_factor_sequence * ...
            PSNR_calc(4,x);
        
        PSNR_calc(3,x)=PSNR_calc(1,x)+confidence_factor_sequence * ...
            PSNR_calc(4,x);
    end
    
    PSNR_calc(1,end)=mean(PSNR_tmp);
    PSNR_calc(4,end)=std(PSNR_tmp);
    
    PSNR_calc(2,end)=PSNR_calc(1,end)-confidence_factor_total * ...
        PSNR_calc(4,end);
    
    PSNR_calc(3,end)=PSNR_calc(1,end)+confidence_factor_total * ...
        PSNR_calc(4,end);
    
    PSNR_calc=10*log10(PSNR_calc);
    
    %SSIM Calculation
    SSIM_tmp=SSIM(1,:)-SSIM(2,:);
    SSIM_tmp=SSIM_tmp(2:2:end);
    
    for x=1:number_of_sequences;
        SSIM_calc(1,x)=mean(SSIM_tmp((x-1)*frames_pr_sequence+1 : ...
            x*frames_pr_sequence));
        
        SSIM_calc(4,x)=std(SSIM_tmp((x-1)*frames_pr_sequence+1 : ...
            x*frames_pr_sequence));
        
        SSIM_calc(2,x)=SSIM_calc(1,x)- ...
            confidence_factor_sequence * SSIM_calc(4,x);
        
        SSIM_calc(3,x)=SSIM_calc(1,x)+...
            confidence_factor_sequence * SSIM_calc(4,x);
    end
    
    SSIM_calc(1,end)=mean(SSIM_tmp);
    SSIM_calc(4,end)=std(SSIM_tmp);
    
    SSIM_calc(2,end)=SSIM_calc(1,end)- ...
        confidence_factor_total * SSIM_calc(4,end);
    
    SSIM_calc(3,end)=SSIM_calc(1,end)+ ...
        confidence_factor_total * SSIM_calc(4,end);
    
    %Write CSV-file
    if (CSV==1)
        fw = fopen(strcat(folder,filename), 'w');
        
        fprintf(fw,'Description:;');
        fprintf(fw,file_descripion);
        fprintf(fw,'\n\n');
        fprintf(fw,'PSNR\n');
        fprintf(fw,'Sequence;Lower (dB);Mean (dB);Upper (dB)\n');
        
        for x=1:number_of_sequences
            fprintf(fw,'%s',sequence_names(x,:));   fprintf(fw,';');
            fprintf(fw,'%f',PSNR_calc(2,x));        fprintf(fw,';');
            fprintf(fw,'%f',PSNR_calc(1,x));        fprintf(fw,';');
            fprintf(fw,'%f',PSNR_calc(3,x));        fprintf(fw,'\n');
        end
        fprintf(fw,'Total;');
        fprintf(fw,'%f',PSNR_calc(2,end));      fprintf(fw,';');
        fprintf(fw,'%f',PSNR_calc(1,end));      fprintf(fw,';');
        fprintf(fw,'%f',PSNR_calc(3,end));      fprintf(fw,'\n');
        
        
        fprintf(fw,'\nSSIM\n');
        fprintf(fw,'Sequence;Lower;Mean;Upper\n');
        for x=1:number_of_sequences
            fprintf(fw,'%s', sequence_names(x,:));  fprintf(fw,';');
            fprintf(fw,'%f',SSIM_calc(2,x));        fprintf(fw,';');
            fprintf(fw,'%f',SSIM_calc(1,x));        fprintf(fw,';');
            fprintf(fw,'%f',SSIM_calc(3,x));        fprintf(fw,'\n');
        end
        fprintf(fw,'Total;');
        fprintf(fw,'%f',SSIM_calc(2,end));      fprintf(fw,';');
        fprintf(fw,'%f',SSIM_calc(1,end));      fprintf(fw,';');
        fprintf(fw,'%f',SSIM_calc(3,end));      fprintf(fw,'\n');
        
        fclose(fw);
    end
    
    %Create and save errorbar plot of SSIM difference
    if (SSIMerrorbar==1)
        errorbar(1:number_of_sequences, SSIM_calc(1,1:end-1), ...
            SSIM_calc(2,1:end-1)-SSIM_calc(1,1:end-1), ...
            SSIM_calc(3,1:end-1)-SSIM_calc(1,1:end-1), 'o');
        hold on;
        grid on;
        plot(0:number_of_sequences+1,zeros(1,number_of_sequences+2),...
            'color', 'black');
        set(gca,'XTickLabel',{' ',sequence_names,' '});
        set(gca,'FontSize',14);
        set(gcf,'Color',[1.0 1.0 1.0]);
        set(gcf,'Position',[100,100,1024,768]);
        ylabel('SSIM difference');
        frame=getframe(gcf);
        [X,map]=frame2im(frame);
        close;
        imwrite(X,strcat(folder,'SSIMdiff.png'));
        
    end
    
    %Create and save errorbar plot of PSNR difference
    if (PSNRerrorbar==1)
        errorbar(1:number_of_sequences, PSNR_calc(1,1:end-1),...
            PSNR_calc(2,1:end-1)-PSNR_calc(1,1:end-1), ...
            PSNR_calc(3,1:end-1)-PSNR_calc(1,1:end-1), 'o');
        hold on;
        grid on;
        plot(0:number_of_sequences+1,zeros(1,number_of_sequences+2),...
            'color', 'black');
        set(gca,'XTickLabel',{' ',sequence_names,' '});
        set(gca,'FontSize',14);
        set(gcf,'Color',[1.0 1.0 1.0]);
        set(gcf,'Position',[100,100,1024,768]);
        ylabel('PSNR (dB) difference');
        frame=getframe(gcf);
        [X,map]=frame2im(frame);
        close;
        imwrite(X,strcat(folder,'PSNRdiff.png'));
        
    end
    
end
end

