function [out] = wavelet_decomp(input_I,input_P)
input_I=strcat(input_I,'.pgm');
input_P=strcat(input_P,'.pgm');

%Read image files
I=(imread(input_I));
P=(imread(input_P));

%Normalization
I=im2double(I);
P=im2double(P);

%Level offset
I=I-0.5;
P=P-0.5;

%Filters in JPEG 2000
Lo_D=[0.0267 -0.0168 -0.0782 0.2668 0.6029 0.2668 -0.0782 -0.0168 ...
    0.0267];
Hi_D=[0.0912 -0.0575 -0.5912 1.1150 -0.5912 -0.0575 0.0912];
 
%Calculate the 2-level Wavelet transform
[scaledI, verticalI, horizontalI, diagonalI]=dwt2(I,Lo_D,Hi_D);

[scaledI2, verticalI2, horizontalI2, diagonalI2]= ...
    dwt2(scaledI, Lo_D,Hi_D);

[scaledP, verticalP, horizontalP, diagonalP]= ...
    dwt2(P, Lo_D, Hi_D);

[scaledP2, verticalP2, horizontalP2, diagonalP2]= ...
    dwt2(scaledP, Lo_D,Hi_D);

%Concatenate matrix so total mean, max, min, and std can be calculated
scaledI=catmat(scaledI);
verticalI=catmat(verticalI);
horizontalI=catmat(horizontalI);
diagonalI=catmat(diagonalI);

scaledI2=catmat(scaledI2);
verticalI2=catmat(verticalI2);
horizontalI2=catmat(horizontalI2);
diagonalI2=catmat(diagonalI2);

scaledP=catmat(scaledP);
verticalP=catmat(verticalP);
horizontalP=catmat(horizontalP);
diagonalP=catmat(diagonalP);

scaledP2=catmat(scaledP2);
verticalP2=catmat(verticalP2);
horizontalP2=catmat(horizontalP2);
diagonalP2=catmat(diagonalP2);

%Calculate Mean, STD, Min, Mac, Range and Entropy for all sub bands.

strings='   Mean       STD       Min       Max       Range     Entropy';
disp(strcat('Original image:',input_I));
disp(strcat('Residue image:',input_P));

disp('----------------------------1LL-------------------------------');
disp(strings);

mxI=max((scaledI));
mxP=max((scaledP));
miI=min((scaledI));
miP=min((scaledP));

disp([mean((scaledI)),std((scaledI)),miI,mxI,mxI-miI, ...
    entropy(scaledI)]);

disp([mean((scaledP)),std((scaledP)),miP,mxP,mxP-miP, ...
    entropy(scaledP2)]);

disp('----------------------------2LL-------------------------------');
disp(strings);

mxI=max((scaledI2));
mxP=max((scaledP2));
miI=min((scaledI2));
miP=min((scaledP2));

I_RANGE=(mxI-miI)*0.0625;
P_RANGE=(mxP-miP)*0.0625;

I_ENT=entropy(scaledI2)*0.0625;
P_ENT=entropy(scaledP2)*0.0625;
STD_ratio=0.0625*(std((scaledP2))/std((scaledI2)));

disp([mean((scaledI2)),std((scaledI2)),miI,mxI,mxI-miI, ...
    entropy(scaledI2)]);

disp([mean((scaledP2)),std((scaledP2)),miP,mxP,mxP-miP, ...
    entropy(scaledP2)]);

disp('----------------------------1HL-------------------------------');
disp(strings);

mxI=max((verticalI));
mxP=max((verticalP));
miI=min((verticalI));
miP=min((verticalP));

I_RANGE=I_RANGE+(mxI-miI)*0.25;
P_RANGE=P_RANGE+(mxP-miP)*0.25;

I_ENT=I_ENT+entropy(verticalI)*0.25;
P_ENT=P_ENT+entropy(verticalP)*0.25;
STD_ratio=STD_ratio + 0.25*(std((verticalP))/std((verticalI)));

disp([mean((verticalI)),std((verticalI)),miI,mxI,mxI-miI, ...
    entropy(verticalI)]);

disp([mean((verticalP)),std((verticalP)),miP,mxP,mxP-miP, ...
    entropy(verticalP)]);

disp('----------------------------2HL-------------------------------');
disp(strings);

mxI=max((verticalI2));
mxP=max((verticalP2));
miI=min((verticalI2));
miP=min((verticalP2));

I_RANGE=I_RANGE+(mxI-miI)*0.0625;
P_RANGE=P_RANGE+(mxP-miP)*0.0625;

I_ENT=I_ENT+entropy(verticalI2)*0.0625;
P_ENT=P_ENT+entropy(verticalP2)*0.0625;
STD_ratio=STD_ratio + 0.0625*(std((verticalP2))/ std((verticalI2)));

disp([mean((verticalI2)),std((verticalI2)),miI,mxI,mxI-miI, ...
    entropy(verticalI2)]);

disp([mean((verticalP2)),std((verticalP2)),miP,mxP,mxP-miP, ...
    entropy(verticalP2)]);

disp('----------------------------1LH-------------------------------');
disp(strings);

mxI=max((horizontalI));
mxP=max((horizontalP));
miI=min((horizontalI));
miP=min((horizontalP));

I_RANGE=I_RANGE+(mxI-miI)*0.25;
P_RANGE=P_RANGE+(mxP-miP)*0.25;

I_ENT=I_ENT+entropy(horizontalI)*0.25;
P_ENT=P_ENT+entropy(horizontalP)*0.25;

STD_ratio=STD_ratio + 0.25*(std((horizontalP))/ std((horizontalI)));

disp([mean((horizontalI)),std((horizontalI)),miI,mxI,mxI-miI, ...
    entropy(horizontalI)]);

disp([mean((horizontalP)),std((horizontalP)),miP,mxP,mxP-miP, ...
    entropy(horizontalP)]);

disp('----------------------------2LH-------------------------------');
disp(strings);

mxI=max((horizontalI2));
mxP=max((horizontalP2));
miI=min((horizontalI2));
miP=min((horizontalP2));

I_RANGE=I_RANGE+(mxI-miI)*0.0625;
P_RANGE=P_RANGE+(mxP-miP)*0.0625;

I_ENT=I_ENT+entropy(horizontalI2)*0.0626;
P_ENT=P_ENT+entropy(horizontalP2)*0.0625;

STD_ratio=STD_ratio + ...
    0.0625*(std((horizontalP2))/ std((horizontalI2)));

disp([mean((horizontalI2)),std((horizontalI2)),miI,mxI,mxI-miI, ...
    entropy(horizontalI2)]);

disp([mean((horizontalP2)),std((horizontalP2)),miP,mxP,mxP-miP, ...
    entropy(horizontalP2)]);

disp('----------------------------1HH-------------------------------');
disp(strings);

mxI=max((diagonalI));
mxP=max((diagonalP));
miI=min((diagonalI));
miP=min((diagonalP));

I_RANGE=I_RANGE+(mxI-miI)*0.25;
P_RANGE=P_RANGE+(mxP-miP)*0.25;

I_ENT=I_ENT+entropy(diagonalI)*0.25;
P_ENT=P_ENT+entropy(diagonalP)*0.25;

STD_ratio=STD_ratio + 0.25*(std((diagonalP))/ std((diagonalI)));

disp([mean((diagonalI)),std((diagonalI)),miI,mxI,mxI-miI, ...
    entropy(diagonalI)]);

disp([mean((diagonalP)),std((diagonalP)),miP,mxP,mxP-miP, ...
    entropy(diagonalP)]);

disp('----------------------------2HH-------------------------------');
disp(strings);

mxI=max((diagonalI2));
mxP=max((diagonalP2));
miI=min((diagonalI2));
miP=min((diagonalP2));

I_RANGE=I_RANGE+(mxI-miI)*0.0625;
P_RANGE=P_RANGE+(mxP-miP)*0.0625;

I_ENT=I_ENT+entropy(diagonalI2)*0.0625;
P_ENT=P_ENT+entropy(diagonalP2)*0.0625;

STD_ratio=STD_ratio + 0.0625*(std((diagonalP2))/ std((diagonalI2)));

disp([mean((diagonalI2)),std((diagonalI2)),miI,mxI,mxI-miI, ...
    entropy(diagonalI2)]);

disp([mean((diagonalP2)),std((diagonalP2)),miP,mxP,mxP-miP, ...
    entropy(diagonalP2)]);
disp('--------------------------AVERAGE-----------------------------');
disp('   RANGE I:  RANGE P:   RANGE I/P:');
disp([I_RANGE,P_RANGE,P_RANGE/I_RANGE]);

disp('   ENT.  I:  ENT.  P:   ENROPY I/P:');
disp([I_ENT,P_ENT,P_ENT/I_ENT]);

disp('   STD %:');
disp(STD_ratio);

out=[I_RANGE,P_RANGE,P_RANGE/I_RANGE,I_ENT,P_ENT,P_ENT/I_ENT, ...
    STD_ratio];

end
